package src.calculator.util;

import java.io.*;
import java.text.*;
import java.util.*;


/** Created on June 26, 2004, 5:07 PM
 * 
 * Problem statement  Create a Mortgage Calculator WITH a graphical user interface.
 *
 * Purpose  provide a user-friendly service to potential users for accurately planning mortgage and other types of loans
 *
 * Scope  The mortgage calculator must display the mortgage payment amount given the amount of the mortgage,
 * the term of the mortgage, and the interest rate of the mortgage. This program must have hard code amount = 200,000,
 * term= 30 years, interest rate = 5.75%, monthly payment = formula using math.pow.  Derive amortization table that will
 * generate Payment #, Principle, Monthly Payment, Pay Date (start date 15JUL04), Interim Balance, Interest Paid, and Principle Paid
 * for each payment over the term of the loan and scrolls on the screen, hesitate and then display more of the list.
 *
 * Algorithm  Divide the yearly interest rate of 5.75% by 12 to get monthly interest rate. Multiply the number of years by 12
 * to get the number of months. Formula for monthly payment: Monthly Payment = [P((1+r)^n)*r]/[(1+r^n)-1], where r and n are
 * calculated as above.
 * Formula for interest balance: Interest Balance= Current Principle * e (percentage rate/365 * days between payments.
 *
 * 
 * NETBEANS IDE 3.6*/

//Calculates, formulates, vectors the mortgage calculator and returns the parameter string.
//Provide control access to the variables by using setters and getters.
public class Mortgagecalculator{
    
    
    //define variable
    public static int years;
    public double loanAmount;
    public double interestRate;
    public double monthlyPayment;
    public double balance;
    private static int numberOfMonths, numberOfDays;
    final static double e = 2.7182818284594;
    Vector firstRow = new Vector();
    NumberFormat form = NumberFormat.getInstance();
    
    public void setYears(int inYears)//setter
    {    years=inYears;
    }//end set years
    
    public static int getYears()//getter
    {
        return years;
    }//end get years
    
    public void setLoanAmount(double inLoanAmount)//setter
    {
        loanAmount=inLoanAmount;
    }//end set loanAmount
    
    public double getLoanAmount()//getter
    {
        return loanAmount;
    }//end get loanAmount
    
    public void setInterestRate(double inInterestRate)//setter
    {
        interestRate=inInterestRate;
    }//end set interestRate
    
    public double getInterestRate()//getter
    {
        return interestRate;
    }//end get interestrate
    
    private int getDaysBetweenPayments(){
        int daysBetweenPayments = 30;
        return daysBetweenPayments;
    }
    //Calculate interim Balance
    private double calculateInterimBalance(double principle){
        double dailyRate = interestRate/365;
        int days = getDaysBetweenPayments();
        double exponent = dailyRate*days;
        double base = Math.pow(e, exponent);
        double interimBalance = principle*(1-base);
        return interimBalance;
    }
    //Calculate interest Paid
    private double calculateInterestPaid(double interimBalance, double previousBalance){
        double interestPaid = interimBalance - previousBalance;
        return interestPaid;
    }
    //Calculate principle Paid
    private double calculatePrinciplePaid(double interestPaid){
        double principlePaid = monthlyPayment - interestPaid;
        return principlePaid;
    }
    //Calculate new Principle
    private double calculateNewPrinciple(double principlePaid, double previousPrinciple){
        double newPrinciple = previousPrinciple-principlePaid;
        return newPrinciple;
    }
    //Calculate monthly Payment
    public void calculateMonthlyPayment(){
        numberOfMonths = years*12;
        numberOfDays = years*365;
        //Divide the yearly interest rate of 5.75% by 12 to get monthly ineterst rate
        // Multiply the number of years by 12 to get the number of months
        //Formula for monthly payment: monthlyPayment = [P((1+r)^n)*r]/[(1+r^n)-1], where r and n are calculated as above.
        monthlyPayment = loanAmount*Math.pow(1 + interestRate/12, numberOfMonths) * (interestRate/12)/(Math.pow(1 + interestRate/12, numberOfMonths) -1);
     }
    //Return Mortgage Information, Loan Amount: 200,000, Interest Rate: .0575, Term 360 months and monthly Payment
    public String toString(){//strings are used to store text and variables
        //2 decimal places
        form.setMaximumFractionDigits(2);
        form.setMinimumFractionDigits(2);
        return "Mortgage Information" + "\n"+ "Loan Amount: $" + form.format(loanAmount) + "\n" + "Interest Rate: " + interestRate + "%" + "\n" + "Term: "
        + (years*12) + " months" + "\n" + "Monthly Payment: $" + form.format(monthlyPayment);//Using the round to two decimal places
    }
        //Drive Vector  
        public void deriveMatrix(){
              
              int counter = 1;
              double Balance;
              double Interest_Payment;
              double Interest_Paid;
              double Principle_Paid;
              String payDate = "15JUL04";
              while(counter <= numberOfMonths){
                  //Payment# is the month number
                  //Principle is constant at $200,000
                  //Monthly Payment is calculated from calculateMonthlyPayment
                  //Pay Date is fixed at 15JUL04
                  //Interest Payment = loanAmount*interestRate/12;
                  //Interest Paid = Interest Payment
                  //Principle Paid = Principle - Balance
                  Balance = Math.round((loanAmount*(1+interestRate/12) - monthlyPayment)*100.00/100.00); //Calculation of Balance
                  Interest_Payment = Math.round((loanAmount*interestRate/12)*100.00/100.00); //Calculation of Interest
                  Interest_Paid = Interest_Payment;
                  Principle_Paid = loanAmount - Balance;
                  //Define date
                  MortgageCalendar calendar = new MortgageCalendar();
                  calendar.advanceDate(counter);
                  payDate = calendar.getDate();   
                   
                  //Define the rows
                  Row rows = new Row(counter, loanAmount, monthlyPayment, payDate, Balance, Interest_Paid, Principle_Paid);
                  firstRow.addElement(rows);
                  counter++;
                  loanAmount = Balance;
              }//end of while(counter <= numberOfMonths)
          }
          //print schedule and scroll
          public void printMatrix(){
              System.out.println();
              System.out.println("Pay#    Principle       MonthlyPayment  PayDate         Balance         IntPaid         PrinPaid");
              int scrollCounter, counter = 0;
              while (counter < numberOfMonths){
                  scrollCounter = 0;
                  while(scrollCounter<30 && counter < numberOfMonths){
                      System.out.println("" + (counter+1) + "\t" +firstRow.elementAt(counter));
                      counter++;
                      scrollCounter++;
                  }//end of while(scrollCounter<30)
                  System.out.println("Press Enter to continue..");
                 //A stream is an abstraction of the continuous one-way flow of data
                  BufferedReader br = new BufferedReader(new InputStreamReader(System.in), 1);
                  String str = "";
                  //try and catch exceptions
                  try {
                      str = br.readLine();
                  }
                  catch (IOException ioe) {
                      System.out.println(ioe);}
              }//end of while (counter <= numberOfMonths)
          }
}//closing bracket


